unit DlgAddEdit;

{-------------------------------------------------------------------------------
  ExtRes Creator V1.0
  Copyright 2003 Tumbleweeds ICT. All rights reserved 
--------------------------------------------------------------------------------
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

  Get the latest version of Extres at www.extres.org.
--------------------------------------------------------------------------------
  Author(s): Patrick van Kemenade
  Revision info :  09-10-2003  Created
-------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Mask, Buttons;

type
  TAddEditDlg = class(TForm)
    GroupBox: TGroupBox;
    lblKey: TLabel;
    MemoResource: TMemo;
    btnOk: TButton;
    btnCancel: TButton;
    lblResTxt: TLabel;
    EditKey: TEdit;
    OpenDialog: TOpenDialog;
    SpBtnInsert: TSpeedButton;
    lblInsert: TLabel;
    procedure RadioButtonPathClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditKeyKeyPress(Sender: TObject; var Key: Char);
    procedure SpBtnInsertClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Key : string;
    Text : string;
    EditMode : boolean;
  end;

  function EscapeControlchars(text : string):string;
  function UnEscapeControlchars(text : string):string;

var
  AddEditDlg: TAddEditDlg;

implementation
uses main, extres;
{$R *.dfm}

procedure TAddEditDlg.FormCreate(Sender: TObject);
begin
  Key := '';
  Text := '';
  EditMode := false;
end;

procedure TAddEditDlg.FormShow(Sender: TObject);
begin
  EditKey.Text := Key;
  MemoResource.Text := '';
  MemoResource.Text := Text;
  EditKey.SetFocus;
end;

procedure TAddEditDlg.RadioButtonPathClick(Sender: TObject);
begin
  MemoResource.SetFocus;
end;

procedure TAddEditDlg.btnOkClick(Sender: TObject);
var
  a,b,c : boolean;
  lpStrIn, lpStrOut : pChar;
begin
  if EditKey.Text = '' then
  begin
    MessageDlg('Please enter a key.',  mtInformation, [mbOk], 0);
    a := false;
  end
  else if not (EditKey.Text[1] in ['a'..'z','A'..'Z']) then
  begin
    MessageDlg('Keys must begin with an alphabetic character (a..z, A..Z).',  mtInformation, [mbOk], 0);
    a := false;
  end
  else
  begin
    Key := EditKey.Text;
    a := true;
  end;

  if not EditMode and not frmMain.KeyUnique(EditKey.Text) then
  begin
    MessageDlg('Error, Key is not unique, please try again.',  mtError, [mbOk], 0);
    b := false;
  end
  else
    b := true;

  if MemoResource.Text = '' then
  begin
    MessageDlg('Please enter your text in the resource box.',  mtInformation, [mbOk], 0);
    c := false;
  end
  else
  begin
    lpStrIn := AllocMem(Length(MemoResource.Text)+1);
    strpcopy(lpStrIn, MemoResource.Text);
    lpStrOut := _extResSerialize(lpStrIn);
    text := string(lpStrOut);
    _extResFree(lpStrOut);
    FreeMem(lpStrIn);
    c := true;
  end;

  if a and b and c then
  begin
    ModalResult := mrOk;
    AddEditDlg.Hide;
  end;
end;


function EscapeControlchars(text : string):string;
var
  i : integer;
begin
  result := '';
  for i := 1 to Length(text) do
  begin
    case ord(text[i]) of
     1..8 : result := result + '\0' + IntToStr(ord(text[i]));
     9   : result := result + '\t';
     10  : result := result + '\n';
     13  : ;
     11,12,14..31 : result := result + '\' + IntToStr(ord(text[i]));
     92 : result := result + '\\';
    else
      result := result + text[i];
    end;
  end;
end;

function UnEscapeControlchars(text : string):string;
var
  escapeCode,
  haveFirstDigit : boolean;
  i, digit : integer;
begin
  result := '';
  escapeCode := false;
  haveFirstDigit := false;
  digit := 0;
  for i := 1 to Length(text) do
  begin
    if escapeCode then
    begin
      case text[i] of
       '0'..'9' : begin
                    digit := digit + strToInt(text[i]);
                    if haveFirstDigit then
                    begin
                      result := result + Chr(digit);
                      escapeCode := false;
                    end
                    else
                      haveFirstDigit := true;
                  end;
       't' :  begin
                result := result + #9;
                escapeCode := false;
              end;
       'n' :  begin
                result := result + #13#10;
                escapeCode := false;
              end;
       '\' :  begin
                result := result + '\';
                escapeCode := false;
              end;
      end;
    end
    else if text[i] = '\' then
    begin
      escapeCode := true;
      haveFirstDigit := false;
      digit := 0;
    end
    else
      result := result + text[i];
  end;
end;

procedure TAddEditDlg.EditKeyKeyPress(Sender: TObject; var Key: Char);
begin
  if ((Ord(Key) > 31) and (Ord(Key) < 48)) or
     ((Ord(Key) > 57) and (Ord(Key) < 65)) or
     ((Ord(Key) > 90) and (Ord(Key) < 95)) or
     ((Ord(Key) > 95) and (Ord(Key) < 97)) or
     (Ord(Key) > 122)then
    Key := Char(0);
  if ((Ord(Key) > 96) and (Ord(Key) < 123)) then  // set lowercase characters to uppercase
    Key := Char(Ord(Key) - 32);
end;

procedure TAddEditDlg.SpBtnInsertClick(Sender: TObject);
var
   fileData : TStringList;
begin
  if OpenDialog.Execute then
  begin
    fileData := TStringList.Create;        // Create  TSTringList object
    fileData.LoadFromFile(OpenDialog.FileName);  // Load from file
    MemoResource.Text := fileData.Text;
    fileData.Free;
  end;

end;

end.
